const {
  IRTCStreamEventType,
  JuphoonWebRTCConference,
  StompSocketAuthType,
  ConferenceEventType,
  SimpleRoleCompose,
  SimpleSVCResolution,
  StorageManagerInstance,
  WebMediaDevicesInstance,
  IRTCCoreEventType,
  PictureSize,
  ConferenceVideoMergeMode,
} = JRTCSDK;

const sdkVersionInfo = JuphoonWebRTCConference.getVersion();
document.querySelector('.version-info').textContent = sdkVersionInfo.version + '-' + sdkVersionInfo.gitHashShort + '-' + sdkVersionInfo.buildTime;

const app = Vue.createApp({
  data() {
    return {
      PictureSize,
      StompSocketAuthType,
      ConferenceVideoMergeMode,
      client: {},
      environmentSettings: {},
      joinSettings: {},
      callSettings: {},
      WebMediaDevicesInstance: {},
      volumeInfo: {
        localVolume: 0,
        remoteVolume: 0
      },
      errorInfo: {
        showError: false,
        message: ''
      }
    };
  },
  created() {
    WebMediaDevicesInstance.refresh()
      .then(() => this.WebMediaDevicesInstance = WebMediaDevicesInstance);

    this.reset();
    const settingFormData = StorageManagerInstance.get('settingFormData_webrtc-conference-sdk');

    if (settingFormData) {
      Object.entries(settingFormData)
        .forEach((entity) => this[entity[0]] = entity[1]);
    }

    window.onbeforeunload = (ev) => {
      const {environmentSettings, joinSettings} = this;
      /**
       * 重置不需要保存的参数
       */
      environmentSettings.showReason = false;
      environmentSettings.environmentConfig.token = undefined;
      joinSettings.showReason = false;

      StorageManagerInstance.set('settingFormData_webrtc-conference-sdk', {
        environmentSettings,
        joinSettings
      });

      ev.preventDefault();
      ev.returnValue = '';
      return '';
    };
  },
  unmounted() {
  },
  methods: {
    setConfig() {
      JuphoonWebRTCConference.setConfig(this.environmentSettings.environmentConfig);
    },
    getToken() {
      this.environmentSettings.environmentConfig.authType = StompSocketAuthType.REDIS;

      this.setConfig();
      JuphoonWebRTCConference.getToken()
        .then((token) => this.environmentSettings.environmentConfig.token = token)
        .catch((reason) => this.handleError(reason));
    },
    login() {
      new Promise((resolve, reject) => {
        try {
          if (!this.client.state) {
            this.client = JuphoonWebRTCConference.createConferenceClient(this.environmentSettings.environmentConfig.appKey, this.environmentSettings.environmentConfig.token);
          }
        } catch (e) {
          reject(e);
        }
        this.client.addEventListeners(
          [ConferenceEventType.CONFERENCE_PARTICIPANT_JOIN, ConferenceEventType.CONFERENCE_PARTICIPANT_UPDATE, ConferenceEventType.CONFERENCE_PARTICIPANT_LEFT, ConferenceEventType.CLIENT_STATE_CHANGE],
          (ev) => this.$forceUpdate()
        );
        this.client.login({thirdAlive: this.environmentSettings.environmentConfig.thirdAlive ? 1 : 0})
          .then(resolve)
          .catch(reject);
      })
        .catch((reason) => this.handleError(reason));

    },
    handleError(reason) {
      console.error(reason.message, reason.stack);
      this.errorInfo.message = reason.message;
      this.errorInfo.showError = true;
    },
    join() {
      this.client.join(this.joinSettings.conferenceNumber, this.joinSettings.joinParam)
        .then(() => (this.joinSettings.joinParam.video === false) ? this.client.startMicrophone() : this.client.startCameraVideo())
        .then((stream) => stream.start('local-stream', document))
        .then(() => this.client.startVideo())
        .then((stream) => stream.start('remote-stream', document))
        .then(() => {
          this.client.localStream.addEventListener(IRTCStreamEventType.AUDIO_VOLUME_CHANGE, (ev) => this.volumeInfo.localVolume = ev.message.data.volume);
          this.client.remoteStream.addEventListener(IRTCStreamEventType.AUDIO_VOLUME_CHANGE, (ev) => this.volumeInfo.remoteVolume = ev.message.data.volume);
        })
        .then(() => this.handleShowStats())
        .catch((reason) => {
          this.joinSettings.reason = reason.message;
          this.joinSettings.showReason = true;
        });
    },
    logout() {
      this.client.logout();
    },
    leave() {
      this.client.leave()
        .then(() => this.handleShowStats());
    },
    reset() {
      this.environmentSettings = {
        environmentConfig: {
          address: 'https://videobu.juphoon.cloud:10047',
          appKey: '05c97ce0d0a84d2592334093',
          token: '',
          authType: StompSocketAuthType.REDIS,
          thirdAlive: false
        },
        sendOnlineMessageParam: {
          toUserId: '',
          content: '',
        },
        roomId: undefined,
        showReason: false,
        reason: undefined
      };
      this.joinSettings = {
        conferenceNumber: '',
        joinParam: {
          displayName: '',
          conferencePassword: '123456',
          enableRecord: false,
          capacity: 6,
          role: SimpleRoleCompose.NORNAL,
          svcResolution: SimpleSVCResolution.HD,
          wholeRatio: 0.5625,
          isZanTong: false,
          video: true,
        },
        showReason: false,
        reason: undefined
      };
      this.callSettings = {
        cameraConstraints: {},
        sendMessageParam: {
          type: '',
          toUserId: '',
          content: '',
        },
        inviteSipUserParam: {
          calleeNumber: '',
          coreNetwork: '',
          sbcIp: '',
          sbcPort: '',
          userToUser: '',
          callerNumber: '',
        },
        remoteRecordParam: {
          recordResolution: PictureSize.LARGE,
          mergeMode: ConferenceVideoMergeMode.INTELLEGNCE,
          videoWidth: 640,
          videoHeight: 360,
          frameRate: 18,
          bitrate: 0,
          fileName: '',
        },
        showStats: false,
        statsInfo: '',
        showReason: false,
        reason: undefined
      };
    },
    handleCameraChanged() {
      this.client.switchCamera(this.callSettings.cameraConstraints);
    },
    triggerUploadVideoStream() {
      this.client.localStream.videoState ? this.client.disableUploadVideoStream() : this.client.enableUploadVideoStream();
    },
    triggerUploadAudioStream() {
      this.client.localStream.audioState ? this.client.disableUploadAudioStream() : this.client.enableUploadAudioStream();
    },
    sendOnlineMessage() {
      const {toUserId, content} = this.environmentSettings.sendOnlineMessageParam;
      this.client.sendOnlineMessage(toUserId, content)
        .catch((reason => this.handleError(reason)));
    },
    sendMessage() {
      const {type, content, toUserId} = this.callSettings.sendMessageParam;

      this.client.sendMessage(type, content, toUserId)
        .catch((reason => this.handleError(reason)));
    },
    query() {
      this.client.query(this.environmentSettings.roomId)
        .then((data) => this.handleError({message: JSON.stringify(data.data, undefined, 2)}))
        .catch((reason) => this.handleError(reason));
    },
    handleShowStats() {
      this.callSettings.showStats = !this.callSettings.showStats;

      this.client.core.getStats(this.callSettings.showStats ? 1000 : 0);
      this.client.core.addEventListener(IRTCCoreEventType.STATS, (ev) => {
        this.callSettings.statsInfo = JSON.stringify(ev.message.data.stats, undefined, 2);
      });
    },
    inviteSipUser() {
      this.client.inviteSipUser(this.callSettings.inviteSipUserParam)
        .catch((reason => this.handleError(reason)));
    },
    startRemoteRecord() {
      this.client.startRemoteRecord(this.callSettings.remoteRecordParam);
    },
    stopRemoteRecord() {
      this.client.stopRemoteRecord();
    },
  }
})
  .mount('#app');
